package org.apache.bcel.classfile;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;

public final class ModuleProvides implements Cloneable, Node
{
    private final int providesIndex;
    private final int providesWithCount;
    private final int[] providesWithIndex;
    
    ModuleProvides(final DataInput file) throws IOException {
        this.providesIndex = file.readUnsignedShort();
        this.providesWithCount = file.readUnsignedShort();
        this.providesWithIndex = new int[this.providesWithCount];
        for (int i = 0; i < this.providesWithCount; ++i) {
            this.providesWithIndex[i] = file.readUnsignedShort();
        }
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitModuleProvides(this);
    }
    
    public ModuleProvides copy() {
        try {
            return (ModuleProvides)this.clone();
        }
        catch (final CloneNotSupportedException ex) {
            return null;
        }
    }
    
    public void dump(final DataOutputStream file) throws IOException {
        file.writeShort(this.providesIndex);
        file.writeShort(this.providesWithCount);
        for (final int entry : this.providesWithIndex) {
            file.writeShort(entry);
        }
    }
    
    @Override
    public String toString() {
        return "provides(" + this.providesIndex + ", " + this.providesWithCount + ", ...)";
    }
    
    public String toString(final ConstantPool constantPool) {
        final StringBuilder buf = new StringBuilder();
        final String interfaceName = constantPool.constantToString(this.providesIndex, (byte)7);
        buf.append(Utility.compactClassName(interfaceName, false));
        buf.append(", with(").append(this.providesWithCount).append("):\n");
        for (final int index : this.providesWithIndex) {
            final String className = constantPool.getConstantString(index, (byte)7);
            buf.append("      ").append(Utility.compactClassName(className, false)).append("\n");
        }
        return buf.substring();
    }
}
